@@ -1,7 +1,8 @@
 # -*- coding: utf-8 -*-
 import json
+import traceback
 from django.db import IntegrityError
-from flask import request, logging
+from flask import request, logging, redirect
 
 from flask.ext import restful
 
@@ -9,6 +10,7 @@ from pagador import MeioDePagamentoNaoEncontrado, MeioDePagamentoNaoDefinido
 from pagador.configuracao.models import FormaDePagamento, FormaDePagamentoConfiguracao
 from pagador.envio.requisicao import EnviarPedido
 from pagador.extensao.base import PluginNaoEncontrado
+from pagador.retorno.registro import RegistrarPagamento
 
 
 logger = logging.getLogger("API-PAGADOR")
@@ -22,65 +24,116 @@ class HelloWorld(restful.Resource):
 class FormasPagamentoResource(restful.Resource):
     def get(self):
         conta_id = request.args.get('conta_id', None)
+        indice = request.args.get('indice', 1)
         com_configuracao = False
         if conta_id:
             com_configuracao = True
             formas_pagamento = FormaDePagamento.configuracoes_ativas(conta_id=conta_id)
         else:
             formas_pagamento = FormaDePagamento.objects.filter(ativo=True)
-        return {"formas_pagamento": [forma.to_dict(com_configuracao=com_configuracao) for forma in formas_pagamento]}
+        return {"formas_pagamento": [forma.to_dict(com_configuracao=com_configuracao, indice=indice) for forma in formas_pagamento]}
 
 
 class PagamentoBaseResource(restful.Resource):
     def __init__(self):
         self.pagamento_id = None
+        self.conta_id = None
         self._forma_de_pagamento = None
 
+    @property
+    def dados(self):
+        dados = {}
+        for arg in request.args:
+            dados[arg] = request.args[arg]
+        return dados
+
     def retorno(self, pagamento_id=None):
         if pagamento_id:
             self.pagamento_id = pagamento_id
         if self.retorno_tem_erro:
-            status = self.forma_de_pagamento["status"]
-            del self.forma_de_pagamento["status"]
-            return self.forma_de_pagamento, status
-        return self.forma_de_pagamento, 200
+            status = self.forma_de_pagamento_to_dict["status"]
+            del self.forma_de_pagamento_to_dict["status"]
+            return self.forma_de_pagamento_to_dict, status
+        return self.forma_de_pagamento_to_dict, 200
 
     @property
     def retorno_tem_erro(self):
-        return "erro" in self.forma_de_pagamento
+        return "erro" in self.forma_de_pagamento_to_dict
 
     @property
     def forma_de_pagamento(self):
         if not self._forma_de_pagamento:
             try:
-                self._forma_de_pagamento = FormaDePagamento.objects.get(id=self.pagamento_id, ativo=True).to_dict(com_configuracao=False)
+                self._forma_de_pagamento = FormaDePagamento.objects.get(id=self.pagamento_id, ativo=True)
             except FormaDePagamento.DoesNotExist:
-                self._forma_de_pagamento = {"erro": u"Não foi encontrada forma de pagamento ativa com id {}".format(self.pagamento_id), "status": 404}
+                self._forma_de_pagamento = None
         return self._forma_de_pagamento
 
+    @property
+    def forma_de_pagamento_to_dict(self):
+        if not self._forma_de_pagamento:
+            try:
+                self._forma_de_pagamento = FormaDePagamento.objects.get(id=self.pagamento_id, ativo=True)
+                return self._forma_de_pagamento.to_dict(com_configuracao=False)
+            except FormaDePagamento.DoesNotExist:
+                return {"erro": u"Não foi encontrada forma de pagamento ativa com id {}".format(self.pagamento_id), "status": 404}
+        return self._forma_de_pagamento.to_dict(com_configuracao=False)
+
 
 class FormaPagamentoResource(PagamentoBaseResource):
     def get(self, pagamento_id):
         return self.retorno(pagamento_id)
 
 
+class PagamentoConfiguracaoBaseResource(PagamentoBaseResource):
+
+    @property
+    def chave(self):
+        raise NotImplementedError
+
+    @property
+    def valor(self):
+        raise NotImplementedError
+
+    @property
+    def pagamento_configuracao(self):
+        return FormaDePagamentoConfiguracao.objects.get(
+            conta_id=self.conta_id,
+            forma_pagamento_id=self.pagamento_id
+        )
+
+    def get(self, pagamento_id, conta_id):
+        self.pagamento_id = pagamento_id
+        self.conta_id = conta_id
+        if self.retorno_tem_erro:
+            return self.retorno()
+        try:
+            return {self.chave: self.valor}
+        except (MeioDePagamentoNaoDefinido, MeioDePagamentoNaoEncontrado, PluginNaoEncontrado), ex:
+            logger.error(u"Erro de plugin em {}: {}".format(self.chave, ex))
+            return {"erro": u"pagador: {}".format(ex)}, 500
+        except IntegrityError, ex:
+            logger.error(u"Erro de banco em {}: {}".format(self.chave, ex))
+            return {"erro": u"Erro no banco de dados: {}".format(ex)}, 500
+
+
 class FormaPagamentoConfiguracaoResource(PagamentoBaseResource):
     def get(self, pagamento_id, conta_id):
         self.pagamento_id = pagamento_id
-        pagamento = self.forma_de_pagamento
         if self.retorno_tem_erro:
             return self.retorno()
         try:
             pagamento_configuracao, criado = FormaDePagamentoConfiguracao.objects.get_or_create(
                 conta_id=conta_id,
-                forma_pagamento_id=pagamento["id"]
+                forma_pagamento_id=pagamento_id
             )
-            #FIXME: Melhorar isso. Tornar o Resource menos conhecedor dessa regra
             return pagamento_configuracao.to_dict()
         except (MeioDePagamentoNaoDefinido, MeioDePagamentoNaoEncontrado, PluginNaoEncontrado), ex:
+            logger.error(u"Erro de plugin em {}: {}".format("pagamento configuracao", ex))
             return {"erro": u"pagador: {}".format(ex)}, 500
         except IntegrityError, ex:
-            return {"erro": "Erro no banco de dados: {}".format(ex)}, 500
+            logger.error(u"Erro de banco em {}: {}".format("pagamento configuracao", ex))
+            return {"erro": u"Erro no banco de dados: {}".format(ex)}, 500
 
     #FIXME: ALERTA!!! ALERTA!!! SEM TESTES!!! ALERTA!!! ALERTA!!!
     def post(self, pagamento_id, conta_id):
@@ -94,45 +147,71 @@ class FormaPagamentoConfiguracaoResource(PagamentoBaseResource):
         except (MeioDePagamentoNaoDefinido, MeioDePagamentoNaoEncontrado, PluginNaoEncontrado), ex:
             return {"erro": u"pagador: {}".format(ex)}, 500
         except IntegrityError, ex:
-            return {"erro": "Erro no banco de dados: {}".format(ex)}, 500
+            return {"erro": u"Erro no banco de dados: {}".format(ex)}, 500
 
 
 #FIXME: ALERTA!!! ALERTA!!! SEM TESTES!!! ALERTA!!! ALERTA!!!
-class FormaPagamentoScriptResource(PagamentoBaseResource):
-    def get(self, pagamento_id, conta_id):
-        self.pagamento_id = pagamento_id
-        pagamento = self.forma_de_pagamento
-        if self.retorno_tem_erro:
-            return self.retorno()
-        try:
-            pagamento_configuracao = FormaDePagamentoConfiguracao.objects.get(
-                conta_id=conta_id,
-                forma_pagamento_id=pagamento["id"]
-            )
-            return {"scripts": pagamento_configuracao.script.to_dict()}
-        except (MeioDePagamentoNaoDefinido, MeioDePagamentoNaoEncontrado, PluginNaoEncontrado), ex:
-            return {"erro": u"pagador: {}".format(ex)}, 500
-        except IntegrityError, ex:
-            return {"erro": "Erro no banco de dados: {}".format(ex)}, 500
+class FormaPagamentoScriptResource(PagamentoConfiguracaoBaseResource):
+
+    @property
+    def chave(self):
+        return "scripts"
+
+    @property
+    def valor(self):
+        return self.pagamento_configuracao.script.to_dict()
 
 
 #FIXME: ALERTA!!! ALERTA!!! SEM TESTES!!! ALERTA!!! ALERTA!!!
-class FormaPagamentoSelecaoResource(PagamentoBaseResource):
+class FormaPagamentoSelecaoResource(PagamentoConfiguracaoBaseResource):
+    @property
+    def chave(self):
+        return "scripts"
+
+    @property
+    def valor(self):
+        return self.pagamento_configuracao.selecao.to_dict()
+
+
+#FIXME: ALERTA!!! ALERTA!!! SEM TESTES!!! ALERTA!!! ALERTA!!!
+class FormaPagamentoIntegracaoResource(PagamentoConfiguracaoBaseResource):
+    @property
+    def chave(self):
+        return "aplicacao"
+
+    @property
+    def valor(self):
+        return {"url": self.pagamento_configuracao.instalador.url_ativador(self.dados)}
+
+    def delete(self, pagamento_id, conta_id):
+        self.pagamento_id = pagamento_id
+        self.conta_id = conta_id
+        try:
+            self.pagamento_configuracao.desinstalar()
+            return {"resultado": "OK"}, 200
+        except Exception, ex:
+            return {"erro": u"Ocorreu um erro durante a remoção do aplicativo. Por favor, tente novamente."}, 500
+            #PARA DEBUG
+            # stack = traceback.format_exc()
+            # return {"erro": u"Servidor: {}\n\n{}".format(ex, stack)}, 500
+
+
+#FIXME: ALERTA!!! ALERTA!!! SEM TESTES!!! ALERTA!!! ALERTA!!!
+class FormaPagamentoInstalacaoResource(PagamentoConfiguracaoBaseResource):
     def get(self, pagamento_id, conta_id):
         self.pagamento_id = pagamento_id
-        pagamento = self.forma_de_pagamento
-        if self.retorno_tem_erro:
-            return self.retorno()
+        self.conta_id = conta_id
+        mensagem = None
+        next_url = self.dados["next_url"]
         try:
-            pagamento_configuracao = FormaDePagamentoConfiguracao.objects.get(
-                conta_id=conta_id,
-                forma_pagamento_id=pagamento["id"]
-            )
-            return {"scripts": pagamento_configuracao.selecao.to_dict()}
-        except (MeioDePagamentoNaoDefinido, MeioDePagamentoNaoEncontrado, PluginNaoEncontrado), ex:
-            return {"erro": u"pagador: {}".format(ex)}, 500
-        except IntegrityError, ex:
-            return {"erro": "Erro no banco de dados: {}".format(ex)}, 500
+            self.pagamento_configuracao.instalar(self.dados)
+        except Exception, ex:
+            mensagem = u"Erro: {}".format(ex)
+        if mensagem:
+            if '?' in next_url:
+                return redirect(u"{}&mensagem={}".format(next_url, mensagem), code=301)
+            return redirect(u"{}?mensagem={}".format(next_url, mensagem), code=301)
+        return redirect(next_url, code=301)
 
 
 #FIXME: ALERTA!!! ALERTA!!! SEM TESTES!!! ALERTA!!! ALERTA!!!
@@ -142,7 +221,7 @@ class FormaPagamentoConfiguracaoEnviarResource(PagamentoBaseResource):
         if request.data:
             dados = json.loads(request.data)
         self.pagamento_id = pagamento_id
-        pagamento = self.forma_de_pagamento
+        pagamento = self.forma_de_pagamento_to_dict
         if self.retorno_tem_erro:
             return self.retorno()
         try:
@@ -155,24 +234,27 @@ class FormaPagamentoConfiguracaoEnviarResource(PagamentoBaseResource):
         except (MeioDePagamentoNaoDefinido, MeioDePagamentoNaoEncontrado, PluginNaoEncontrado), ex:
             return {"erro": u"pagador: {}".format(ex)}, 500
         except IntegrityError, ex:
-            return {"erro": "Erro no banco de dados: {}".format(ex)}, 500
+            return {"erro": u"Erro no banco de dados: {}".format(ex)}, 500
         except Exception, ex:
-            return {"erro": u"Servidor: {}".format(ex)}, 500
+            import traceback
+            return {"erro": u"Servidor: {} - {}".format(ex, traceback.format_exc())}, 500
 
 
-class FormaPagamentoAtivacao(PagamentoBaseResource):
-    def get(self, pagamento_id, conta_id):
+class FormaPagamentoRetornoResource(PagamentoBaseResource):
+    def get(self, pagamento_id, tipo=None):
         self.pagamento_id = pagamento_id
-        pagamento = self.forma_de_pagamento
         if self.retorno_tem_erro:
             return self.retorno()
         try:
-            pagamento_configuracao = FormaDePagamentoConfiguracao.objects.get(
-                conta_id=conta_id,
-                forma_pagamento_id=pagamento["id"]
-            )
-            return {"ativacao": pagamento_configuracao.ativador.to_dict()}
+            registro = RegistrarPagamento(self.forma_de_pagamento.codigo, self.dados)
+            resultado = registro.processar(tipo)
+            if "redirect" in resultado:
+                return redirect(resultado["redirect"], code=301)
+            return resultado
         except (MeioDePagamentoNaoDefinido, MeioDePagamentoNaoEncontrado, PluginNaoEncontrado), ex:
             return {"erro": u"pagador: {}".format(ex)}, 500
         except IntegrityError, ex:
-            return {"erro": "Erro no banco de dados: {}".format(ex)}, 500
+            return {"erro": u"Erro no banco de dados: {}".format(ex)}, 500
+        except Exception, ex:
+            # import traceback
+            return {"erro": u"Servidor: {}".format(ex)}, 500
