@@ -5,12 +5,6 @@ import os
 if not 'DJANGO_SETTINGS_MODULE' in os.environ:
     os.environ['DJANGO_SETTINGS_MODULE'] = 'api_pagador.settings'
 
-INSTALLED_APPS = (
-    'repositories.plataforma',
-    'repositories.catalogo',
-    'repositories.configuracao',
-)
-
 SECRET_KEY = 'v6q!c(7t@spf&-4a%(awdesf4&b6(!a8z2^a)%^0#+8a9t(7hs'
 
 HOST = '0.0.0.0'
