@@ -10,4 +10,9 @@ def registrar():
     api.add_resource(recursos.FormaPagamentoConfiguracaoResource, '/forma-pagamento/<int:pagamento_id>/configuracao/<int:conta_id>/')
     api.add_resource(recursos.FormaPagamentoScriptResource, '/forma-pagamento/<int:pagamento_id>/script/<int:conta_id>/')
     api.add_resource(recursos.FormaPagamentoSelecaoResource, '/forma-pagamento/<int:pagamento_id>/selecao/<int:conta_id>/')
+    api.add_resource(recursos.FormaPagamentoIntegracaoResource, '/forma-pagamento/<int:pagamento_id>/integracao/<int:conta_id>/')
+    api.add_resource(recursos.FormaPagamentoInstalacaoResource, '/forma-pagamento/<int:pagamento_id>/instalacao/<int:conta_id>/')
     api.add_resource(recursos.FormaPagamentoConfiguracaoEnviarResource, '/forma-pagamento/<int:pagamento_id>/configuracao/<int:conta_id>/pedido/<int:pedido_numero>/enviar')
+    api.add_resource(recursos.FormaPagamentoRetornoResource,
+                     '/forma-pagamento/<int:pagamento_id>/retorno',
+                     '/forma-pagamento/<int:pagamento_id>/retorno/<string:tipo>/')
