@@ -9,7 +9,7 @@ class Gateways(object):
     pagseguro = 1
     pagamento_digital = 2
     paypal = 3
-    mercado_pago = 4
+    mercado_pago = 11
     koin = 9
 
 
@@ -17,6 +17,10 @@ class FormaDePagamento(models.FormaPagamento, Plugavel):
     class Meta:
         proxy = True
 
+    @property
+    def meio_pagamento(self):
+        return self.codigo
+
     def save(self, *args, **kwargs):
         if self.conta and not self.contrato_id:
             self.contrato_id = self.conta.contrato_id
@@ -31,26 +35,33 @@ class FormaDePagamento(models.FormaPagamento, Plugavel):
         return 'img/bandeiras/{}-logo.png'.format(self.codigo)
 
     @property
-    def html_escolha(self):
-        try:
-            return self.plugin_configuracao.MeioPagamentoSelecao()
-        except AttributeError:
-            return ""
+    def bancos_ativos_na_conta(self):
+        bancos_ativos = self.bancos.filter(conta=self.conta, ativo=True)
+        if bancos_ativos:
+            return [{"id": banco.id, "nome": unicode(banco)} for banco in bancos_ativos]
+        return None
 
     @property
-    def em_uso_para_o_usuario(self):
+    def configurado(self):
         try:
+            try:
+                bancos = self.bancos.all()[0]
+            except IndexError:
+                bancos = None
             return (self.pagamento_configuracao_usuario or
                     self.pagamento_configuracao_senha or
                     self.pagamento_configuracao_assinatura or
                     self.pagamento_configuracao_token or
                     self.pagamento_configuracao_codigo_autorizacao or
-                    self.bancos.all or self.pagamento_configuracao_json)
+                    bancos or self.pagamento_configuracao_json)
         except AttributeError:
             return None
 
     @classmethod
-    def configuracoes_ativas(cls, conta_id=None):
+    def configuracoes_ativas(cls, conta_id, pagamento_id=None):
+        parametros = [conta_id]
+        if pagamento_id:
+            parametros.append(pagamento_id)
         return cls.objects.raw(
             """SELECT
                     pg.pagamento_id,
@@ -68,10 +79,10 @@ class FormaDePagamento(models.FormaPagamento, Plugavel):
                     pgc.conta_id
                FROM configuracao.tb_pagamento as pg
                LEFT OUTER JOIN configuracao.tb_pagamento_configuracao as pgc on (pgc.pagamento_id = pg.pagamento_id AND pgc.conta_id=%s)
-               WHERE pg.pagamento_ativado = true;""", [conta_id]
+               WHERE pg.pagamento_ativado = true{};""".format((" AND pg.pagamento_id=%s" if pagamento_id else '')), parametros
         )
 
-    def to_dict(self, com_configuracao=True):
+    def to_dict(self, com_configuracao=True, indice=None):
         forma_pagamento = {
             "id": self.id,
             "codigo": self.codigo,
@@ -79,12 +90,14 @@ class FormaDePagamento(models.FormaPagamento, Plugavel):
             "ativo_no_sistema": self.ativo,
             "imagem_admin": self.imagem_admin,
             "imagem_loja": self.imagem_loja,
-            "html_escolha": self.html_escolha,
             "plano_indice": self.plano_indice,
         }
         if com_configuracao:
             forma_pagamento["ativo_para_a_conta"] = self.pagamento_configuracao_ativo
-            forma_pagamento["configurado"] = self.em_uso_para_o_usuario is not None
+            forma_pagamento["configurado"] = self.configurado is not None
+            if indice:
+                forma_pagamento["habilitado_na_loja"] = forma_pagamento["configurado"] and self.ativo and self.pagamento_configuracao_ativo and self.plano_indice <= indice
+                forma_pagamento["bancos_ativos_na_conta"] = self.bancos_ativos_na_conta
 
         return forma_pagamento
 
@@ -136,12 +149,38 @@ class FormaDePagamentoConfiguracao(models.FormaPagamentoConfiguracao, Plugavel):
         except AttributeError:
             raise PluginNaoEncontrado(u"O meio de pagamento '{}' não possui a classe configuracao.MeioPagamentoSelecao.".format(self.forma_pagamento.codigo))
 
+    def instalar(self, dados):
+        dados_instalacao = self.instalador.dados_de_instalacao(dados)
+        for campo in dados_instalacao.keys():
+            setattr(self, campo, dados_instalacao[campo])
+        self.save()
+
+    def dados_para_instalador(self):
+        dados = {}
+        for campo in self.instalador.campos:
+            dados[campo] = getattr(self, campo, None)
+        return dados
+
+    def atualizar(self):
+        dados = self.dados_para_instalador()
+        dados_atualizacao = self.instalador.dados_de_atualizacao(dados)
+        for campo in dados_atualizacao.keys():
+            setattr(self, campo, dados_atualizacao[campo])
+        self.save()
+
+    def desinstalar(self):
+        dados = self.dados_para_instalador()
+        for campo in dados:
+            setattr(self, campo, None)
+        self.instalador.desinstalar(dados)
+        self.save()
+
     @property
-    def ativador(self):
+    def instalador(self):
         try:
-            return self.plugin_seguranca.Ativador()
-        except AttributeError:
-            raise PluginNaoEncontrado(u"O meio de pagamento '{}' não possui a classe seguranca.Ativador.".format(self.forma_pagamento.codigo))
+            return self.plugin_seguranca.Instalador(id=self.conta_id)
+        except AttributeError, ex:
+            raise PluginNaoEncontrado(u"O meio de pagamento '{}' não possui a classe seguranca.Instalador.".format(self.forma_pagamento.codigo))
 
     @property
     def valores(self):
@@ -159,6 +198,14 @@ class FormaDePagamentoConfiguracao(models.FormaPagamentoConfiguracao, Plugavel):
             #FIXME: Pensar em um modelo melhor para determinar extensoes não obrigatórias
             # raise PluginNaoEncontrado(u"O meio de pagamento '{}' não possui a classe configuracao.MeioPagamentoCadastro.".format(self.forma_pagamento.codigo))
 
+    @property
+    def eh_aplicacao(self):
+        try:
+            return self.plugin_configuracao.eh_aplicacao
+        except AttributeError:
+            return False
+            #FIXME: Pensar em um modelo melhor para determinar extensoes não obrigatórias
+
     def salvar(self, dados):
         self.formulario.define_valores_em_model(self, dados).save()
 
@@ -166,8 +213,14 @@ class FormaDePagamentoConfiguracao(models.FormaPagamentoConfiguracao, Plugavel):
         cadastro = None
         if self.cadastro:
             cadastro = self.cadastro.to_dict()
+        try:
+            forma_pagamento = FormaDePagamento.configuracoes_ativas(self.conta.id, pagamento_id=self.forma_pagamento.id)[0].to_dict()
+        except IndexError:
+            forma_pagamento = {}
         return {
             "cadastro": cadastro,
             "valores": self.valores.to_dict(),
-            "formulario": self.formulario.to_dict()
+            "formulario": self.formulario.to_dict(),
+            "pagamento": forma_pagamento,
+            "eh_aplicacao": self.eh_aplicacao
         }
