@@ -18,6 +18,9 @@ class Credenciador(object):
     def obter_credenciais(self):
         return "user {token} password {senha}".format(self.credenciamento)
 
+    def esta_valido(self):
+        return True
+
 
 class Credenciais(base.Plugavel):
     def __init__(self, meio_pagamento, configuracao_pagamento):
@@ -34,6 +37,15 @@ class Credenciais(base.Plugavel):
                 raise AutenticadorNaoEncontrado(u"O meio de pagamento '{}' não possui a classe Credenciador.".format(self.meio_pagamento))
         return self._credenciador
 
+    def esta_valido(self):
+        return self.credenciador.esta_valido()
+
+    def por_query_string(self, chave="access_token"):
+        try:
+            return "{}={}".format(chave, self.credenciador.obter_credenciais())
+        except AttributeError:
+            raise CredenciadorInvalido(u"O credenciador {}.{} não implementa o método obter_credenciais(credenciamento=None).".format(self.credenciador.__module__, self.credenciador.__class__.__name__))
+
     def por_cabecalho_http(self, cabecalho="Authorization"):
         try:
             return {cabecalho: self.credenciador.obter_credenciais()}
