@@ -4,7 +4,7 @@ DEBUG = False
 
 LIBS_MEIOS_DE_PAGAMENTO = {
     "koin": "pagador_koin.extensao",
-    "mercado_pago": "pagador_mercado_pago.extensao"
+    "mercadopago": "pagador_mercado_pago.extensao"
 }
 
 QUEUE_URL = "http://api.awsli.com.br/v2/"
